package com.sun.jmx.snmp;

import java.net.UnknownHostException;
import java.net.InetAddress;
import java.io.Serializable;

public class SnmpPeer implements Serializable
{
    private static final long serialVersionUID = -5554565062847175999L;
    public static final int defaultSnmpRequestPktSize = 2048;
    public static final int defaultSnmpResponsePktSize = 8192;
    private int maxVarBindLimit;
    private int portNum;
    private int maxTries;
    private int timeout;
    private SnmpPduFactory pduFactory;
    private long _maxrtt;
    private long _minrtt;
    private long _avgrtt;
    private SnmpParams _snmpParameter;
    private InetAddress _devAddr;
    private int maxSnmpPacketSize;
    InetAddress[] _devAddrList;
    int _addrIndex;
    private boolean customPduFactory;
    
    public SnmpPeer(String s) throws UnknownHostException {
        this(s, 161);
    }
    
    public SnmpPeer(InetAddress devAddr, int portNum) {
        this.maxVarBindLimit = 25;
        this.portNum = 161;
        this.maxTries = 3;
        this.timeout = 3000;
        this.pduFactory = new SnmpPduFactoryBER();
        this._snmpParameter = new SnmpParameters();
        this._devAddr = null;
        this.maxSnmpPacketSize = 2048;
        this._devAddrList = null;
        this._addrIndex = 0;
        this.customPduFactory = false;
        this._devAddr = devAddr;
        this.portNum = portNum;
    }
    
    public SnmpPeer(InetAddress devAddr) {
        this.maxVarBindLimit = 25;
        this.portNum = 161;
        this.maxTries = 3;
        this.timeout = 3000;
        this.pduFactory = new SnmpPduFactoryBER();
        this._snmpParameter = new SnmpParameters();
        this._devAddr = null;
        this.maxSnmpPacketSize = 2048;
        this._devAddrList = null;
        this._addrIndex = 0;
        this.customPduFactory = false;
        this._devAddr = devAddr;
    }
    
    public SnmpPeer(String s, int portNum) throws UnknownHostException {
        this.maxVarBindLimit = 25;
        this.portNum = 161;
        this.maxTries = 3;
        this.timeout = 3000;
        this.pduFactory = new SnmpPduFactoryBER();
        this._snmpParameter = new SnmpParameters();
        this._devAddr = null;
        this.maxSnmpPacketSize = 2048;
        this._devAddrList = null;
        this._addrIndex = 0;
        this.customPduFactory = false;
        this.useIPAddress(s);
        this.portNum = portNum;
    }
    
    public final synchronized void useIPAddress(String host) throws UnknownHostException {
        this._devAddr = InetAddress.getByName(host);
    }
    
    public final synchronized String ipAddressInUse() {
        byte[] address = this._devAddr.getAddress();
        return (address[0] & 0xFF) + "." + (address[1] & 0xFF) + "." + (address[2] & 0xFF) + "." + (address[3] & 0xFF);
    }
    
    public final synchronized void useAddressList(InetAddress[] array) {
        this._devAddrList = ((array != null) ? ((InetAddress[])array.clone()) : null);
        this._addrIndex = 0;
        this.useNextAddress();
    }
    
    public final synchronized void useNextAddress() {
        if (this._devAddrList == null) {
            return;
        }
        if (this._addrIndex > this._devAddrList.length - 1) {
            this._addrIndex = 0;
        }
        this._devAddr = this._devAddrList[this._addrIndex++];
        return;
    }
    
    public boolean allowSnmpSets() {
        return this._snmpParameter.allowSnmpSets();
    }
    
    public final InetAddress[] getDestAddrList() {
        return (this._devAddrList == null) ? null : ((InetAddress[])this._devAddrList.clone());
    }
    
    public final InetAddress getDestAddr() {
        return this._devAddr;
    }
    
    public final int getDestPort() {
        return this.portNum;
    }
    
    public final synchronized void setDestPort(int portNum) {
        this.portNum = portNum;
    }
    
    public final int getTimeout() {
        return this.timeout;
    }
    
    public final synchronized void setTimeout(int timeout) {
        if (timeout < 0) {
            throw new IllegalArgumentException();
        }
        this.timeout = timeout;
        return;
    }
    
    public final int getMaxTries() {
        return this.maxTries;
    }
    
    public final synchronized void setMaxTries(int maxTries) {
        if (maxTries < 0) {
            throw new IllegalArgumentException();
        }
        this.maxTries = maxTries;
        return;
    }
    
    public final String getDevName() {
        return this.getDestAddr().getHostName();
    }
    
    @Override
    public String toString() {
        return "Peer/Port : " + this.getDestAddr().getHostAddress() + "/" + this.getDestPort();
    }
    
    public final synchronized int getVarBindLimit() {
        return this.maxVarBindLimit;
    }
    
    public final synchronized void setVarBindLimit(int maxVarBindLimit) {
        this.maxVarBindLimit = maxVarBindLimit;
    }
    
    public void setParams(SnmpParams snmpParameter) {
        this._snmpParameter = snmpParameter;
    }
    
    public SnmpParams getParams() {
        return this._snmpParameter;
    }
    
    public final int getMaxSnmpPktSize() {
        return this.maxSnmpPacketSize;
    }
    
    public final synchronized void setMaxSnmpPktSize(int maxSnmpPacketSize) {
        this.maxSnmpPacketSize = maxSnmpPacketSize;
    }
    
    boolean isCustomPduFactory() {
        return this.customPduFactory;
    }
    
    @Override
    protected void finalize() {
        this._devAddr = null;
        this._devAddrList = null;
        this._snmpParameter = null;
    }
    
    public long getMinRtt() {
        return this._minrtt;
    }
    
    public long getMaxRtt() {
        return this._maxrtt;
    }
    
    public long getAvgRtt() {
        return this._avgrtt;
    }
    
    private void updateRttStats(long minrtt) {
        if (this._minrtt > minrtt) {
            this._minrtt = minrtt;
        }
        else if (this._maxrtt < minrtt) {
            this._maxrtt = minrtt;
        }
        else {
            this._avgrtt = minrtt;
        }
    }
}
